backend_registry = {}


class Registry(type):
    def __new__(mcs, clsname, bases, attrs):
        m = type.__new__(mcs, clsname, bases, attrs)
        backend_registry[clsname] = mcs
        return m


class BaseBackend(object):
    __metaclass__ = Registry

    bucket_prefix = 'ONTHEFLY'

    def __init__(self, options, original=None):
        self.options = options
        self.original = original
        backend_registry[type(self).__name__] = self

    def get_value_from_original_settings(self, name):
        return getattr(self.original, name)

    def get_all_fields(self):
        raise NotImplementedError()

    def add_field(self, name):
        raise NotImplementedError()

    def set_value(self, name, value):
        raise NotImplementedError()

    def get_value(self, name):
        raise NotImplementedError()

    def get_all_values(self):
        raise NotImplementedError()

    def delete(self, name):
        raise NotImplementedError()



import json
from onthelfy.backend import BaseBackend
from redis import Redis


class RedisBackend(BaseBackend):
    def __init__(self, options, **kwargs):
        self.client = Redis.from_url(options['URL'])
        super(RedisBackend, self).__init__(options, **kwargs)

    def get_all_fields(self):
        fields_dumped = self.client.hget(self.bucket_prefix, 'fields')
        return json.loads(fields_dumped) if fields_dumped else []

    def add_field(self, name):
        fields_dumped = self.client.hget(self.bucket_prefix, 'fields')
        fields = json.loads(fields_dumped) if fields_dumped else []
        fields.append(name)
        self.client.hset(
            self.bucket_prefix, 'fields', json.dumps(list(set(fields))))

        original_value = self.get_value_from_original_settings(name)
        self.set_value(name, original_value)

    def set_value(self, name, value):
        self.client.hset(
            self.bucket_prefix, name, json.dumps(value))

    def get_value(self, name):
        dumped = self.client.hget(self.bucket_prefix, name)
        return json.loads(dumped)

    def get_all_values(self):
        fields = self.get_all_fields()
        values = self.client.hmget(
            self.bucket_prefix, fields)
        all_values = {}
        for i, field in enumerate(fields):
            all_values[field] = json.loads(values[i])
        return all_values

    def delete(self, name):
        fields = self.get_all_fields()
        fields.pop(name)
        self.client.hset(
            self.bucket_prefix, 'fields', json.dumps(list(set(fields))))
        self.client.hdel(self.bucket_prefix, name)



from django.apps import AppConfig

from onthelfy.monkey_patch import patch


class OntheflyConfig(AppConfig):
    name = 'onthelfy'
    patched = False

    def ready(self):
        if not self.patched:
            patch()
            self.patched = True



#! /usr/bin/env python
# -*- coding: utf-8 -*-

import os
from setuptools import setup, find_packages
from onthelfy import __version__

README = open(os.path.join(os.path.dirname(__file__), 'README.md')).read()

# allow setup.py to be run from any path
os.chdir(os.path.normpath(os.path.join(os.path.abspath(__file__), os.pardir)))

packages = find_packages()

setup(
    name='django-onthefly',
    version=__version__,
    description='Change Django Settings On the Fly',
    long_description=README,
    url='https://github.com/baranbartu/onthfly',
    download_url='https://github.com/baranbartu/onthfly/tarball/%s' % (
        __version__,),
    author='Baran Bartu Demirci',
    author_email='bbartu.demirci@gmail.com',
    license='MIT',
    keywords='django,settings,change django settings',
    packages=packages,
    install_requires=[
        'Django>=1.7',
        'redis>=2.10.5'
    ]
)
